//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Import/ImportDataUtil.cpp
//! @brief     Implements ImportDataUtil namespace
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Import/ImportDataUtil.h"
#include "Device/Data/Datafield.h"
#include "GUI/Model/Device/RealItem.h"
#include "GUI/Support/IO/ImportDataInfo.h"
#include "GUI/View/Loaders/QREDataLoader.h"
#include <QApplication>
#include <QFile>
#include <QMessageBox>

QString GUI::View::ImportDataUtil::Import1dData(RealItem* realItem,
                                                const IDataLoader* selectedLoader)
{
    const QString fileName = realItem->nativeFileName();
    const std::string fileNameStdString = fileName.toStdString();

    if (selectedLoader == nullptr) {
        IO::Filetype1D ftype = IO::filename2type1D(fileNameStdString);
        if (IO::isCompressed(fileNameStdString) || ftype == IO::bornagain1D) {
            try {
                throw std::runtime_error("Unsupported case");
            } catch (std::exception& ex) {
                // Try with CSV import ? TODO check the logic, simplify
                if (ftype != IO::bornagain1D || IO::isCompressed(fileNameStdString))
                    // import is not possible
                    return QString::fromLatin1(ex.what());
            }
        }
    }

    // -- try with selected CSV loader. If none selected, try with QRE loader

    QFile file(fileName);
    const bool ok = file.open(QFile::ReadOnly | QIODevice::Text);
    if (!ok)
        return "File could not be opened.";

    QByteArray fileContent = file.readAll();
    file.close();

    if (fileContent.isEmpty())
        return "The imported file is empty.";

    if (IO::isCompressed(fileNameStdString)) {
        // #baimport implement decompress
    }

    IDataLoader* loader = nullptr;
    if (selectedLoader == nullptr)
        loader = new QREDataLoader();
    else
        loader = selectedLoader->clone();
    loader->initWithDefaultImportSettings();
    loader->setRealDataItem(realItem);
    realItem->setDataLoader(loader);
    QApplication::setOverrideCursor(Qt::WaitCursor);
    loader->setFileContents(fileContent);
    loader->guessSettings();
    loader->processContents();
    QApplication::restoreOverrideCursor();

    return QString();
}
