//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Loaders/QREDataLoaderResultModel.cpp
//! @brief     Implements class QREDataLoaderResultModel
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Loaders/QREDataLoaderResultModel.h"

QREDataLoaderResultModel::QREDataLoaderResultModel(QREDataLoader::ImportResult* importResult)
    : m_importResult(importResult)
{
}

int QREDataLoaderResultModel::rowCount() const
{
    return m_importResult->lines.size();
}

QString QREDataLoaderResultModel::headerTextOfCalculatedColumn(int column) const
{
    switch (column) {
    case 0:
        return "Q [1/nm]";
    case 1:
        return "R";
    case 2:
        return "sigma_R";
    default:
        return "";
    }
}

int QREDataLoaderResultModel::columnCount(ColumnType type) const
{
    switch (type) {
    case ColumnType::line:
    case ColumnType::processed: {
        const bool showErrorColumn =
            m_importResult->importSettings.columnDefinitions[QREDataLoader::DataType::dR].enabled;
        return showErrorColumn ? 3 : 2;
    }
    default:
        return 0;
    }
}

QString QREDataLoaderResultModel::cellText(ColumnType type, int row, int col) const
{
    if (col < 0 || row < 0 || row >= m_importResult->lines.size())
        return "";

    const bool isSkippedLine = m_importResult->lines[row].first;
    if (isSkippedLine)
        return "";

    switch (type) {

    case ColumnType::processed:
        if (col == 0)
            return (row < m_importResult->qValues.size())
                       ? QString::number(m_importResult->qValues[row])
                       : QString();
        if (col == 1)
            return (row < m_importResult->rValues.size())
                       ? QString::number(m_importResult->rValues[row])
                       : QString();
        if (col == 2)
            return (row < m_importResult->eValues.size())
                       ? QString::number(m_importResult->eValues[row])
                       : QString();
        return "";

    default:
        return "";
    }
}
